 jeefo.register("node_modules/@jeefo/parser/ast_node_definition.js", async (exports, module) => { const __dirname = "node_modules/@jeefo/parser", __filename = "node_modules/@jeefo/parser/ast_node_definition.js"; const require = path => { return jeefo.require(path, __filename, __dirname); }; /* -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
* File Name   : ast_node_definition.js
* Created at  : 2019-01-26
* Updated at  : 2021-01-06
* Author      : jeefo
* Purpose     :
* Description :
_._._._._._._._._._._._._._._._._._._._._.*/
// ignore:start
"use strict";

/* globals*/
/* exported*/

// ignore:end

const for_each      = (await require("@jeefo/utils/object/for_each"));
const Readonly      = (await require("@jeefo/utils/object/readonly"));
const capitalize    = (await require("@jeefo/utils/string/capitalize"));
const extend_member = (await require("@jeefo/utils/class/extend_member"));
const I_AST_Node    = (await require("./i_ast_node"));

var AST_Node_Definition = (function () { 
	/**
     * @param symbol_definition {Object} - Symbol definition
	 */
	function AST_Node_Definition (ast_node_definition) {
        if (typeof ast_node_definition.id !== "string") {
            throw new Error("Invalid argument: ast_node_definition.id");
        }
        if (typeof ast_node_definition.type !== "string") {
            throw new Error("Invalid argument: ast_node_definition.type");
        }
        if (typeof ast_node_definition.precedence !== "number") {
            throw new Error("Invalid argument: ast_node_definition.precedence");
        }
        if (typeof ast_node_definition.is !== "function") {
            throw new Error("Invalid argument: ast_node_definition.is");
        }
        if (typeof ast_node_definition.initialize !== "function") {
            throw new Error("Invalid argument: ast_node_definition.initialize");
        }

        const class_name = ast_node_definition.id.split(' ').map(word => {
            return capitalize(word);
        }).join('');

        // jshint evil:true
        const Node = (new Function("I_AST_Node", `
            function ${class_name} () {
                I_AST_Node.call(this);
                this._id         = "${ast_node_definition.id}";
                this._type       = "${ast_node_definition.type}";
                this._precedence = ${ast_node_definition.precedence};
            }
            ${class_name}.prototype = Object.create(I_AST_Node.prototype);

            return ${class_name};
        `))(I_AST_Node);
        // jshint evil:false

        const proto_readonly    = new Readonly(Node.prototype);
        const instance_readonly = new Readonly(this);
        for_each(ast_node_definition, (key, value) => {
            switch (key) {
                case "id" :
                case "type" :
                case "precedence" :
                    proto_readonly.prop(key, value, false);
                    instance_readonly.prop(key, value);
                    break;
                case "protos" : break;
                default:
                    this[key] = value;
            }
        });

        if (ast_node_definition.protos) {
            for_each(ast_node_definition.protos, (key, value) => {
                extend_member(Node, key, value);
            });
        }

        instance_readonly.prop("Node", Node, false);
	}

	AST_Node_Definition.prototype.generate_new_node = function (parser) {
		const node = new this.Node();
		this.initialize(node, parser.next_token, parser);

		return node;
	};

    AST_Node_Definition.prototype.clone = function () {
        const protos     = {};
        const definition = { protos };

        Object.getOwnPropertyNames(this).forEach(prop => {
            switch (prop) {
                case "Node"        :
                case "constructor" :
                    break;
                default:
                    definition[prop] = this[prop];
            }
        });

        const { prototype } = this.Node;
        Object.getOwnPropertyNames(prototype).forEach(prop => {
            switch (prop) {
                case "id"          :
                case "type"        :
                case "precedence"  :
                case "constructor" :
                    break;
                default:
                    protos[prop] = prototype[prop];
            }
        });

        return new AST_Node_Definition(definition);
    };
AST_Node_Definition.__jeefo_class__ = true; return AST_Node_Definition;}());

module.exports = AST_Node_Definition;
 }); 
//# sourceURL=node_modules/@jeefo/parser/ast_node_definition.js 